/*
   A Program to Convert Wordstar files to HTML format. 

            v. 3, Apr 13, 1999

    syntax is: WS_HTM src dst

              where src, dst are filespecs
  ----------------------------------------------------------------
             environment: MS-DOS, using MIX Power C compiler
             (superset of Microsoft C, Borland Turbo C), however,
             should be extremely portable.
  -----------------------------------------------------------------
             This program is in the public domain, and may be 
             used for any purpose whatsover, without permission 
             or notification, but it is offered "as-is" without 
             any warrentee, express or implied.
  -----------------------------------------------------------------

   To convert Wordstar 3/4 to HTML. I suggest you use Wordstar's 
   internal filters to convert Wordstar 5 and subsequent to Wordstar 4, 
   then use my program to go the rest of the way. 
   
                                   Andrew D. Todd
                                   1249 Pineview Dr., Apt 4
                                   Morgantown, WV 26505

                                   U46A8@WVNVM.WVNET.EDU


*/
#include <stdio.h>
#include <io.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>



main(int argc, char *argv[])
{
  int status, f_in, f_out;
  char e_string[80];

pass these through to parse-buffer
  char input_buffer[10000000];
  int index_buffer;

  
  /*get the file names*/
  if( argc < 3 )
  {
    printf(" \nBoth Source and Destination Files Needed\n");
    _exit(3);
  }
  f_in = open(argv[1], O_RDONLY|O_BINARY);
  if(f_in == -1) 
  {
    printf(" \n%s Does Not Exist\n", argv[1]);
    _exit(4);
  }
  f_out = open(argv[2], O_WRONLY|O_CREAT|O_EXCL|O_BINARY, S_IREAD|S_IWRITE);
  if( f_out == -1) 
  {
    strcpy(&e_string, " \nDestination File ");
    strcat(&e_string,  argv[2]);
    strcat(&e_string, " Invalid\n");
    perror(&e_string);
    _exit(5);
  }
  /*create a header */  
  put_line(f_out, "<HTML>");
  put_line(f_out, "<HEAD>");
  put_line(f_out, "</HEAD>");
  put_line(f_out, "<BODY>");

/* read thefile into a buffer */
   status = read(f_in, &c, 1);
   if(status != 1) break;
    
else 

 -----------------------------------------------------
----------------------------------------------
 
  /* now add the tail and close*/
  put_line(f_out, "</BODY>");
  put_line(f_out, "</HTML>");
  close(f_in);
  close(f_out);
  _exit(0);
}


flip_attr(int *is_attr, int this_attr, int f_out, char *beg_str, char *end_str)    
{
  if(is_attr[this_attr])
  {
    is_attr[this_attr] = 0;
    write(f_out, end_str, strlen(end_str));
  }
  else
  {
    is_attr[this_attr] = 1;
    write(f_out, beg_str, strlen(beg_str));
  }
}
/*--------------------------------------------------------*/
parse_buffer()
{
  int i;
  char curr_c, c_s, next_c, c_after_next;
  
  int is_attr[32];

  int curr_column;

  /* define the names of the text state counters */
  int bold_attr = 0;
  int doub_attr = 1;
  int undl_attr = 2;
  int sup_attr = 3;
  int sub_attr = 4;
  int ital_attr = 5;
  int strikeout_attr = 6;

  /*and zero them*/
  for(i=0;i < 32; i++)
  {
    is_attr[i] = 0;
  }

  curr_column = 0;

  /*and now do the conversion */
  for(index_buffer=0;;)            /* Do over the whole buffer */
  {
    curr_c=input_bufer[index_buffer];
    c_s = curr_c & '\x7f';
    next_c=input_bufer[index_buffer+1];
    c_after_next=input_bufer[index_buffer+2];
    if(curr_c == ¨.¨)
    {
       if(curr_column == 0)/* if it is a dot command*/
       {
          write_s(f_out, ¨WORDSTAR DOT COMMAND: ¨);
          curr_column = 1;
       }
       else /*if itis only a period*/
       {
         send_char_to_html(f_out, curr_c); 
         curr_column++;
       }
    }
    /*
      WordStar ¨dot-commands¨ are user visible, 
            and are fully described in the user manual.
    */
    else if( isprint(curr_c))               /*PRINTABLE*/
    {
       send_char_to_html(f_out, curr_c);  
       curr_column++;
    }
    else if( isprint(c_s))              /* PRINTABLE WITH HIGH BIT */
    {
       send_char_to_html(f_out, c_s); 
       curr_column++;
    }
    else if( curr_c == '\r') 
    {
      write_s(f_out, "\r\n<P>");  /*HARD CARRIAGE RETURN*/
      curr_column = 0;
      for(i=0;i < 32; i++) is_attr[i] = 0;
            /*  makes the assumption that special
                                  attributes dont persist across a 
                                  paragraph break */
    } 
    else if( curr_curr_c == '\x8d') 
         write_s(f_out, "\r\n"); /*SOFT CARRIAGE RETURN*/
    /*
       In Wordstar, a number of Cntrl-bytes toggle the print 
        attribute of text. IN HTML, the attributes have different tags
         to turn them off and on. So we have to keep track of the 
           state of the Wordstar document, 
            and select the appropriate HTML tag. 
    */
    else if( curr_c == '\x02' /*Cntrl-B*/)  
              flip_attr(&is_attr, bold_attr, f_out, "<B>", "</B>");
    else if( curr_c == '\x04' /*Cntrl-D*/)  
              flip_attr(&is_attr, doub_attr, f_out, "<B>", "</B>");
    else if( curr_c == '\x13' /*Cntrl-S*/)  
              flip_attr(&is_attr, undl_attr, f_out, "<U>", "</U>");
    else if( curr_c == '\x14' /*Cntrl-T*/)  
              flip_attr(&is_attr, sup_attr, f_out, "<SUP>", "</SUB>");
    else if( curr_c == '\x16' /*Cntrl-V*/)  
           flip_attr(&is_attr, sub_attr, f_out, "<SUB>", "</SUB>");
    else if( curr_c == '\x18' /*Cntrl-X*/)  
           flip_attr(&is_attr, strikeout_attr, f_out, "<S>", "</S>");
    else if( curr_c == '\x19' /*Cntrl-Y*/)  
           flip_attr(&is_attr, ital_attr, f_out, "<EM>", "</EM>");
    else /* put out the hex code and let the user figure out what it means */
    {
      write_hex_code(f_out, curr_c);
    }
  }
} 
/*------------------------------------------------------------------*/
write_s(int f_out, char *s)
{
}

/*------------------------------------------------------------------*/
write_hex_code(int f_out, char c)
{
  char c1, c2, c1c, c2c;
  char[16] char_rep= ¨[hex code bb]¨;
  c2=c%16;   /*modulo off the high order nybble*/
  c1=(c-c2)/16; /* and subtract to obtain the remainder*/
  if(c1 <= 9) c1c = c1 + '0';
       else c1c = c1 + 'A'-10;
  if(c2 <= 9) c2c = c2 + '0';
       else c2c = c2 + 'A'-10;
   char_rep[11] = c1c;
   char_rep[12] =c2c;
   write_s(f_out, &char_rep);
}


/*------------------------------------------------------------------*/
send_char_to_html(int f_out, char c)
{
 char *send_str;
 if(c == "\"") strcpy(send_str, "&quot;");
 else if(c == "&") strcpy(send_str, "&amp;");
 else if(c == "<") strcpy(send_str, "&lt;");
 else if(c == ">") strcpy(send_str, "&gt;");
 else
 {
   send_str[0] = c;
   send_str[1] = '\0';
 }
 /*always*/  
 write(f_out, send_str, strlen(send_str));
}
/*provides aliasing for source characters which are reserved in HTML*/
/*------------------------------------------------------------------*/

put_line(int f_out, char *line_str)
{
  write(f_out, line_str, strlen(line_str)); 
  write(f_out, "\r\n", 2);
}  

